/* Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include <termios.h>
#ifdef __FreeBSD__
#include <sys/ioctl_compat.h>
#endif /* __FreeBSD__ */
#include <sys/types.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
/* SunOS 4.x gets redefinition errors, but Linux needs this. */
#if !defined(__sun__)
#include <sys/ioctl.h>
#endif
#include <string.h>
#include "mutt.h"
#include "mutt_curses.h"

int ci_any_key_to_continue (const char *s)
{
  struct termios t;
  int f, ch;

  f = open("/dev/tty", O_RDONLY);
#if defined(__FreeBSD__) || defined(_HPUX_SOURCE) || defined(__convex__)
  tcgetattr(f, &t);
#else
  ioctl(f, TCGETS, &t);
#endif
  t.c_lflag &= ~(ICANON | ECHO);
  t.c_cc[VMIN] = 1;
  t.c_cc[VTIME] = 0;
#if defined(__FreeBSD__) || defined(_HPUX_SOURCE) || defined(__convex__)
  tcsetattr(f, TCSADRAIN, &t);
#else
  ioctl(f, TCSETS, &t);
#endif
  if (s) fputs(s, stdout);
  else fputs("Press any key to continue...", stdout);
  fflush(stdout);
  ch = fgetc(stdin);
  fflush(stdin);
  fputc('\n', stdout);
  t.c_cflag |= (ICANON | ECHO);
#if defined(__FreeBSD__) || defined(_HPUX_SOURCE) || defined(__convex__)
  tcsetattr(f, TCSADRAIN, &t);
#else
  ioctl(f, TCSETS, &t);
#endif
  close(f);
  return(ch);
}

int ci_enter_string (char *buffer, int y, int x, int flags, int initch)
{
  int ch;
  short lastchar, i, len, j, beg = 0;
  int pass = flags & CI_password;
  int complete = flags & CI_complete;

  len = COLS - x - 1;

redraw_it:
  move(y,x);
  lastchar = i = strlen(buffer);
  if (!pass && i>0) {
    beg = lastchar - len;
    if (beg < 0)
      beg = 0;
    else if (beg > 0)
      clrtoeol();
    for (j=beg; j<lastchar && j<beg+len; j++) {
      addch(buffer[j]);
    }
  }
  refresh();
  FOREVER {
    if (!initch)
      ch = getch();
    else {
      ch = initch;
      initch = 0;
    }
    dprint(5, (debugfile, "ci_enter_string(): ch = %d.\n", ch));
    if (complete && (ch == ' ' || ch == '\t')) {
      buffer[i]=0;
      mutt_complete(buffer);
      goto redraw_it;
    }
    else if (ch == KEY_BACKSPACE || ch == 127) {
      if (i == 0) {
        beep();
        refresh();
        continue;
      }
      if (i == lastchar) {
        lastchar--;
        i--;
        if (!pass) {
          if (i < beg) {
            move(y,x);
            clrtoeol();
            beg -= len;
            if (beg < 0) beg = 0;
            for(j=beg; j<lastchar && j<beg+len; j++) {
              addch(buffer[j]);
            }
          }
          else {
            move(y,x+i-beg);
            delch();
          }
        }
      }
      else {
        for (j = i; j<lastchar; j++)
          buffer[j-1] = buffer[j];
        i--;
        lastchar--;
        if (!pass) {
          buffer[lastchar] = 0;
          move(y,x+i-beg);
          clrtoeol();   
          for(j=i; j<lastchar && j<beg+len; j++) {
            addch(buffer[j]);
          }
          move(y,x+i-beg);
        }
      }
    }
    else if (ch == ctrl('A') || ch == KEY_HOME) {
      i = 0;
      if (!pass) {
        move(y,x);
        beg = 0;
        if (x + lastchar > len) {
          clrtoeol();
          for(j=0;j<len;j++) {
            addch(buffer[j]);
          }
          move(y,x);
        }
      }
    }
    else if (ch == ctrl('E') || ch == KEY_END) {
      if (!pass) {
        if (lastchar < beg + len)
          move(y,x+lastchar-beg);
        else {
          move(y,x);
          clrtoeol();
          beg = lastchar-len/2;
          for(j=beg; j<lastchar; j++) {
            addch(buffer[j]);
          }
        }
      }
      i = lastchar;
    }
    else if (ch == ctrl('U')) {
      lastchar = i = 0;
      buffer[0] = 0;
      if (!pass) {
        move(y,x);
        clrtoeol();
        beg = 0;
      }
    }
    else if (ch == ctrl('K')) {
      lastchar = i;
      buffer[i] = 0;
      if (!pass) {
        clrtoeol();
      }
    }
    else if (ch == ctrl('B') || ch == KEY_LEFT) {
      if (i == 0)
        beep();
      else {
        i--;
        if (!pass) {
          if (i < beg) {
            beg -= len;
            if (beg < 0) beg = 0;
            move(y,x);
            clrtoeol();
            for(j=beg;j<beg+len;j++) {
              addch(buffer[j]);
            }
          }
          move(y,x+i-beg);
        }
      }
    }
    else if (ch == ctrl('F') || ch == KEY_RIGHT) {
      if (i == lastchar)
        beep();
      else {
        i++;
        if (!pass) {
          if (i >= beg + len) {
            beg = i;
            move(y,x);
            clrtoeol();
            for(j=beg; j<lastchar && j<beg+len; j++) {
              addch(buffer[j]);
            }
            move(y,x);
          }
          else
            move(y,x+i-beg);
        }
      }
    }
    else if (ch == ctrl('D')) {
      for (j=i;j<lastchar;j++)
        buffer[j] = buffer[j+1];
      lastchar--;
      if (!pass) {
        clrtoeol();
        for (j=i;j<lastchar && j<beg+len; j++)
          addch(buffer[j]);
        move(y,x+i-beg);
      }
    }
    else if (ch == ctrl('G')) { /* ABORT */
      buffer[0] = 0;
      return(-1);
    }
    else if (CI_is_return(ch)) {
      buffer[lastchar] = 0;
      return(0);
    }
    else if (isprint(ch)) {
      if (i == lastchar) {
        buffer[i++] = ch;
        lastchar++;
        if (!pass) {
          if (i > beg+len) {
            beg = i - len/2;
            move(y,x);
            clrtoeol();
            for (j=beg;j<lastchar;j++) {
              addch(buffer[j]);
            }
          }
          else {
            addch(ch);
          }
        }
      }
      else {
        for (j=lastchar; j>i; j--)
          buffer[j] = buffer[j-1];
        buffer[i] = ch;
        i++;
        lastchar++;
        if (!pass) {
          if (i > beg + len) {
            move(y,x);
            clrtoeol();
            beg = i-1;
            for(j=beg;j<lastchar && j<beg+len; j++) {
              addch(buffer[j]);
            }
          }
          else {
            clrtoeol();
            for(j=i-1;j<lastchar && j<beg+len; j++) {
              addch(buffer[j]);
            }
          }
          move(y,x+i-beg);
        }
      }
    }
    else
      beep();
    refresh();
  }
}
     
int ci_get_field (const char *field, char *buffer, int initch, int complete)
{
  int ret;
  int len = strlen(field); /* in case field==buffer */

  ClearLine(LINES-2);
  addstr(field);
  refresh();
  ret = ci_enter_string(buffer, LINES-2, len, complete, initch);
  ClearLine(LINES-2);
  return(ret);
}

void ci_clear_error (void)
{
  if (Errorbuf[0]) {
    Errorbuf[0] = 0;
    ClearLine(LINES-1);
    refresh();
  }
}

void ci_edit_file (const char *p)
{
  char cmd[256];

  endwin();
  sprintf(cmd, "%s %s", Editor, p);
  mutt_system(cmd);
}
